/*******************************************************************
 *
 * File: sample.c
 * 
 * Author: Tomasz Stachowiak
 *
 * Description: Sample audio file
 *
 * Date: 07.17.96
 *
 *******************************************************************/


#include <stdio.h>
#include <dmedia/audio.h>
#include "params.h"
#include "gsm.h"
#include "adpcm.h"

#define GSM 1
#define ADPCM 2

void SetInRate(long rate);
void SetOutRate(long rate);

ALport micro, speaker;
ALconfig conf;
void *buf, *comp_buf, *decomp_buf;
int i;
gsm handle;
gsm_signal src[160];
gsm_signal uncomp[160];
gsm_frame comp;
struct adpcm_state state;
char acomp[80];

void main(int argc, char **argv)
{
   int size;
   int code=0;

  buf = malloc(BUF_SIZE);  
  comp_buf = malloc(BLOCK);
  decomp_buf = malloc(BUF_SIZE);  

   if(argc == 2)
      {
	 if(strcmp(argv[1], "gsm") == 0)
	   {
	      code = GSM;
	   };
         if(strcmp(argv[1], "adpcm") == 0)
	   {
	       code = ADPCM;
	    }
      }

  Mic_init();
  Speaker_init();

  handle = gsm_create();

  while(1)
    {
      ALreadsamps(micro, src, 160);
      switch(code)
	{
	   case GSM:
	   gsm_encode(handle, src, comp);
	   gsm_decode(handle, comp, uncomp);
	   break;

	   case ADPCM:
	   state.valprev = 0;
	   state.index = 0;
	   adpcm_coder(src, acomp, 160, &state);
	   state.valprev = 0;
	   state.index = 0;
	   adpcm_decoder(acomp, uncomp, 160, &state);
	   break;

	   default:
	   memcpy(uncomp, src, 320);
	}
      ALwritesamps(speaker, uncomp, 160);
    }

  gsm_destroy(handle);
  free(buf);
  ALcloseport(micro);
  ALcloseport(speaker);
}



Mic_init()
  {
  SetInRate(32000);

  if ((conf = ALnewconfig()) == 0)
    fprintf (stderr, "Config allocation error\n");
  if (ALsetchannels(conf, MONO) < 0)
    fprintf (stderr, "Setting channel error\n");
  if (ALsetqueuesize(conf, QSIZE) < 0)
    fprintf (stderr, "Setting queue size error\n");
  if (ALsetwidth(conf, AL_SAMPLE_16) < 0)
    fprintf (stderr, "Setting sample width error\n");

  if ((micro = ALopenport("microphone", "r", conf)) == 0)
    fprintf (stderr, "Port opening error");
  }


Speaker_init()
  {
  SetOutRate(32000);

  if ((conf = ALnewconfig()) == 0)
    fprintf (stderr, "Config allocation error\n");
  if (ALsetchannels(conf, MONO) < 0)
    fprintf (stderr, "Setting channel error\n");
  if (ALsetqueuesize(conf, QSIZE) < 0)
    fprintf (stderr, "Setting queue size error\n");
  if (ALsetwidth(conf, AL_SAMPLE_16) < 0)
    fprintf (stderr, "Setting sample width error\n");

  if ((speaker = ALopenport("speaker", "w", conf)) == 0)
    fprintf (stderr, "Port opening error");
  }


void SetInRate(long rate)
  {
     long buf[2];

     buf[0] = AL_INPUT_RATE;
     buf[1] = rate;

     ALsetparams(AL_DEFAULT_DEVICE, buf, 4);
  }

void SetOutRate(long rate)
  {
     long buf[2];

     buf[0] = AL_OUTPUT_RATE;
     buf[1] = rate;

     ALsetparams(AL_DEFAULT_DEVICE, buf, 4);
  }
